local metadata = {
"## Interface: FS15 1.3.0.0PublicBeta1 1.3PublicBeta1",
"## Title: RabbitShelterPlaceable",
"## Notes: Script for FSM-Team placeable Rabbit Shelter",
"## Author: Marhu",
"## Version: 1.1.0-29",
"## Date: 28.07.2015",
"## Web: http://marhu.net"
}

local ModDir = g_currentModDirectory
RabbitShelterPlaceable = {};
RabbitShelterPlaceable_mt = Class(RabbitShelterPlaceable, Placeable);
InitObjectClass(RabbitShelterPlaceable, "RabbitShelterPlaceable");

function RabbitShelterPlaceable:new(isServer, isClient, customMt)
  
	local mt = customMt;
    if mt == nil then
         mt = RabbitShelterPlaceable_mt;
    end;
	
	local self = Placeable:new(isServer, isClient, mt);
	registerObjectClassName(self, "RabbitShelterPlaceable");
			
	self.incomePerHour = 0
	
	return self;
end;
 
function RabbitShelterPlaceable:load(xmlFilename, x,y,z, rx,ry,rz, initRandom)

	if not RabbitShelterPlaceable:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, initRandom) then
		return false;
	end;
	
	self.xmlFile = loadXMLFile("TempXML", xmlFilename);
	
	local difficultyMultiplier = 2^(g_currentMission.missionStats.difficulty - 3)
	self.incomePerHour = difficultyMultiplier*Utils.getNoNil(getXMLFloat(self.xmlFile, "placeable.incomePerHour"), 100);
	
	return true;
end;

function RabbitShelterPlaceable:finalizePlacement(x, y, z, rx, ry, rz, initRandom)
	
	RabbitShelterPlaceable:superClass().finalizePlacement(self)
	
	if self.isClient then
		local AniAnimalFile = getXMLString(self.xmlFile, "placeable.AniAnimal#File");
		local AnimalPoseGroupIndex = getXMLString(self.xmlFile, "placeable.AnimalPoseGroup#Index");
		
		if AniAnimalFile ~= nil and AnimalPoseGroupIndex ~= nil then
			local BaseAnimals = {}
			local file = Utils.getFilename(AniAnimalFile, ModDir);
			local AnimalsI3d = Utils.loadSharedI3DFile(file);
			link(getRootNode(), AnimalsI3d);
			
			local i = 0;
			while true do
				local AnimalKey = string.format("placeable.AniAnimal.Animal(%d)", i);
				if not hasXMLProperty(self.xmlFile, AnimalKey) then
					break;
				end;
				local temp = {}
				local rootIndex = getXMLString(self.xmlFile, AnimalKey.."#index");
				temp.root = Utils.indexToObject(AnimalsI3d,rootIndex)	
				temp.jointIndex = getXMLString(self.xmlFile, AnimalKey.."#jointIndex");
				temp.MeshIndex = getXMLString(self.xmlFile, AnimalKey.."#MeshIndex");
				temp.ClipName = getXMLString(self.xmlFile, AnimalKey.."#ClipName");
				temp.RNDmin = Utils.getNoNil(getXMLFloat(self.xmlFile, AnimalKey.."#RNDmin"), 0);
				temp.RNDmax = Utils.getNoNil(getXMLFloat(self.xmlFile, AnimalKey.."#RNDmax"), 0);
				table.insert(BaseAnimals,temp)
				i = i + 1;
			end;
			
			local AnimalPoseGroup = Utils.indexToObject(self.nodeId, AnimalPoseGroupIndex);
			
			self.AniAnimals = {}
			local numAnimals = table.getn(BaseAnimals);
			local numChildren = getNumOfChildren(AnimalPoseGroup);
			for i = numChildren, 1, -1 do
				local posnode = getChildAt(AnimalPoseGroup,i-1)
				local pos = {getWorldTranslation(posnode)}
				local rot = {getWorldRotation(posnode)}
				local RndRot = Utils.getNoNil(getUserAttribute(posnode, "UseRndRot"), true)
				if RndRot == true then
					rot[2] = math.rad(math.random(0, 359))
				end
			
				local num = Utils.getNoNil(getUserAttribute(posnode, "Animal"),(i % numAnimals))+1
				local animal = {}
				animal.animal = clone(BaseAnimals[num].root,true)
				animal.joint = Utils.indexToObject(animal.animal,BaseAnimals[num].jointIndex)
				animal.mesh = Utils.indexToObject(animal.animal,BaseAnimals[num].MeshIndex)
				animal.RNDmin = BaseAnimals[num].RNDmin;
				animal.RNDmax = BaseAnimals[num].RNDmax;
				
				setTranslation(animal.mesh,unpack(pos))
				setRotation(animal.mesh,unpack(rot))
				
				animal.Animi = getAnimCharacterSet(animal.joint);
				animal.Clip = getAnimClipIndex(animal.joint,BaseAnimals[num].ClipName)
				assignAnimTrackClip(animal.Animi, 0, animal.Clip);
				setAnimTrackLoopState(animal.Animi, 0, false);
				setAnimTrackSpeedScale(animal.Animi, 0, 1);
				
				animal.time = math.random(animal.RNDmin, animal.RNDmax)
				
				table.insert(self.AniAnimals,animal);
			end
			
			local num = table.getn(BaseAnimals);
			for i = num, 1, -1 do
				delete(BaseAnimals[i].root)
				BaseAnimals[i] = nil;
			end
		end
	end		

	local AreaIndex = getXMLString(self.xmlFile, "placeable.Area#Index");
	if AreaIndex ~= nil and AreaIndex ~= "" then         
		local AreaStart = Utils.indexToObject(self.nodeId, AreaIndex);
		if AreaStart ~= nil then
			self.Area = {}
			self.Area.start  = AreaStart;
			self.Area.width	 = getChildAt(AreaStart,0);
			self.Area.height = getChildAt(AreaStart,1);
			local x,_,z = getWorldTranslation(self.Area.start);
			local x1,_,z1 = getWorldTranslation(self.Area.width);
			local x2,_,z2 = getWorldTranslation(self.Area.height);
			
			Utils.updateDestroyCommonArea(x, z, x1, z1, x2, z2);
		end;
	end;
	
	self.RabbitShelterPlaceableDirtyFlag = self:getNextDirtyFlag();
	
	g_currentMission.environment:addHourChangeListener(self)
	
	delete(self.xmlFile);
	
	return true;
end;

function RabbitShelterPlaceable:getSaveAttributesAndNodes(nodeIdent)
	
	local attributes, nodes = RabbitShelterPlaceable:superClass().getSaveAttributesAndNodes(self, nodeIdent);
	  
    return attributes,nodes;
end
function RabbitShelterPlaceable:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	
	if not RabbitShelterPlaceable:superClass().loadFromAttributesAndNodes(self, xmlFile, key, resetVehicles) then
		return false
	end
	
	return true;
end

function RabbitShelterPlaceable:writeAllStream(streamId, connection)

end;
function RabbitShelterPlaceable:readAllStream(streamId, connection)
	
end; 
function RabbitShelterPlaceable:writeStream(streamId, connection)
	RabbitShelterPlaceable:superClass().writeStream(self, streamId, connection)
	self:writeAllStream(streamId, connection)
end
function RabbitShelterPlaceable:readStream(streamId, connection)
	RabbitShelterPlaceable:superClass().readStream(self, streamId, connection)
	self:readAllStream(streamId, connection)
end;
function RabbitShelterPlaceable:writeUpdateStream(streamId, connection, dirtyMask)
	RabbitShelterPlaceable:superClass().writeUpdateStream(self, streamId, connection, dirtyMask);
	self:writeAllStream(streamId, connection)
end;
function RabbitShelterPlaceable:readUpdateStream(streamId, timestamp, connection)
	RabbitShelterPlaceable:superClass().readUpdateStream(self, streamId, timestamp, connection);
	self:readAllStream(streamId, connection)
end;

function RabbitShelterPlaceable:delete()
	unregisterObjectClassName(self)
	g_currentMission.environment:removeHourChangeListener(self)
	
	if self.AniAnimals then
		local num = table.getn(self.AniAnimals);
		for i = num, 1, -1 do
			delete(self.AniAnimals[i].animal)
			self.AniAnimals[i] = nil;
		end
	end
	RabbitShelterPlaceable:superClass().delete(self)
end;

function RabbitShelterPlaceable:update(dt)
	
end;

function RabbitShelterPlaceable:updateTick(dt)
	if self.AniAnimals then
		for i = 1, table.getn(self.AniAnimals) do
			local a = self.AniAnimals[i];
			a.time = a.time - dt
			if a.time <= 0 then
				if not isAnimTrackEnabled(a.Animi, 0) then
					enableAnimTrack(a.Animi, 0);
				end
				if getAnimTrackTime(a.Animi, 0) > getAnimClipDuration(a.Animi, 0) then
					setAnimTrackTime(a.Animi, 0, 0, false);
					a.time = math.random(a.RNDmin, a.RNDmax)
				end;
			elseif isAnimTrackEnabled(a.Animi, 0) and getAnimTrackTime(a.Animi, 0) > getAnimClipDuration(a.Animi, 0) then
				disableAnimTrack(a.Animi, 0);
			end;
		end;
	end
end;
 
function RabbitShelterPlaceable:hourChanged()
	if self.isServer then
		g_currentMission:addSharedMoney(self.incomePerHour, "other")
	end
end

registerPlaceableType("RabbitShelterPlaceable", RabbitShelterPlaceable);

 

